@@ -12,6 +12,7 @@ if(con.__mode__=="debug"):
     prefix = ""
 
 
+exec("from "+prefix+"starfishXfn import setTHforPlot")
 
 #from starfishX.starfishXfn import loadHistData
 exec("from "+prefix+"starfishXfn import loadHistData")
@@ -144,6 +145,8 @@ exec("from "+prefix+"EYG import utilityTwinXPlot")
 
 
 exec("from "+prefix+"starfishXfn import RelativeStrengthRank")
+exec("from "+prefix+"starfishXfn import RelativeStrength_hist")
+
 
 exec("from "+prefix+"Elder import elderSafezone")
 
@@ -169,7 +172,19 @@ exec("from "+prefix+"utility import *")
 exec("from "+prefix+"risk_kit import *")
 
 
+exec("from "+prefix+"nvdr import getNVDRVolume")
+exec("from "+prefix+"nvdr import reduceXtickfrequency") 
 
-__version__ = 0.155504
+
+exec("from "+prefix+"news_api import news_api")
+exec("from "+prefix+"news_api import NewsTypeMatch")
+
+exec("from "+prefix+"sentimentAnalysis import sentimentAnalysis")
+
+
+exec("from "+prefix+"ShareholdersNow import ShareholdersNow")
+exec("from "+prefix+"ShareholdersNow import ShareholdersHist")
+
+__version__ = 0.155522
 
 __doc__ = """starfishX Version จัดเตรียมข้อมูลสำหรับประมวลผลการลงทุนในตลาดหุ้นไทย """
\ No newline at end of file
