@@ -36,6 +36,10 @@ def str2Date(dt):
  return dt
 
 def listSSF(where=""):
+ """
+ ดูข้อมูลของ SSF ในซีรีย์ที่มีสถานะอยู่
+ where : (string) คัดกองเฉพาะหุ้นแม่ที่สนใจ
+ """  
  #url = "https://www.tfex.co.th/tfex/dailyMarketReport.html?locale=en_US&marketListId=SF&instrumentId=&selectedDate=C&periodView=A"
  url = "https://www.tfex.co.th/tfex/dailyMarketReport.html?locale=en_US&marketListId=SF&instrumentId=&selectedDate=C&periodView=A"
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
@@ -92,6 +96,10 @@ def OpenInterestContractsProcess(symbol):
   return df
 
 def OpenInterestContracts(listSymbol):
+   """
+   listSymbol : (list) ส่งค่าซีรีย์ SSF ที่ต้องการ ตัวอย่าง การดึงค่าจาก sx.listSSF(where='KCE')
+                df = sx.OpenInterestContracts(["KCEU20","KCEZ20","KCEH21","KCEM21"])
+   """
    if(type(listSymbol)==str):
       return OpenInterestContractsProcess(listSymbol)
    if(type(listSymbol)==list): 
