@@ -132,7 +132,7 @@ def loadHistData(symbol,start,end="",Volume=False,OHLC=False,US=False,col_clean=
       ##### ตรวจว่าเป็นดัชนีไหม 18 มิถุนายน 2020 
       if(symbol[0]=="^"):
         symbol = symbol.replace("^","")
-        df = loadHistDataIndex(symbol,start)
+        df = loadHistDataIndex(symbol,start,end)
         return df #ถ้าเป็น str แล้วเป็นชนิด index คืนค่าออกไปเลย -- 18 มิถุนายน 2020
         ###################################
         ###################################
@@ -161,12 +161,17 @@ def loadHistData(symbol,start,end="",Volume=False,OHLC=False,US=False,col_clean=
          symbol = [symbol]
       
       symbol = tmpsymbol
+ 
     #จบส่วนของเช็ค str
 
  
     if(qs!=0): #คือมีชุด query String ของหุ้นรายตัว*  ต่อมาสำเร็จ กรณีไม่สำเร็จเช่น ["^set","^set50"]
      try:
       df = loaddata.get(qs,start=start,end=endDate)
+      
+      if(len(symbol_stock_only)==0):
+          symbol_stock_only = symbol
+        
       df.columns = symbol_stock_only #symbol
       if(col_clean):
             df = col_cleanFn(df)
@@ -175,7 +180,7 @@ def loadHistData(symbol,start,end="",Volume=False,OHLC=False,US=False,col_clean=
        return 0   
  
     if(contain_index):
-       rpIndex = loadHistDataIndex(symbol_index_only,start=start)
+       rpIndex = loadHistDataIndex(symbol_index_only,start=start,end=end)
        if(qs!=0):
          tmp = pd.merge(df,rpIndex,on='Date',how='inner')
          if(col_clean):
@@ -202,11 +207,19 @@ def col_cleanFn(df):
 ############## update 18 มีถุนายน 2020 ############# 
 ############## รุ่นแก้ โหลดข้อมูลดัชนี #############   
 from datetime import datetime as dt
-def loadHistDataIndex(symbol,start):
+def loadHistDataIndex(symbol,start,end):
  if(type(symbol)==str):
-   symbol = symbol.upper()
+   if(not("sSET" in symbol)):
+      symbol = symbol.upper()
  if(type(symbol)==list):
-   symbol = [i.upper() for i in symbol]   
+   #symbol = [i.upper() for i in symbol] 
+   symbol_ = []
+   for i in symbol:
+     if(not("sSET" in i)):
+       symbol_.append(i.upper())
+     else:
+       symbol_.append(i)  
+   symbol = symbol_    
 
  #url
  url = "https://www.set.or.th/static/mktstat/setindex.csv"   
@@ -232,9 +245,9 @@ def loadHistDataIndex(symbol,start):
  df = df.set_index("Date")
     
  if(type(symbol)==str):
-   return df[[symbol]][df.index>=start]
+   return df[[symbol]][(df.index>=start) & (df.index<=end)]
  if(type(symbol)==list):
-   return df[symbol][df.index>=start]
+   return df[symbol][(df.index>=start) & (df.index<=end)]
 
 ############## end รุ่นแก้ โหลดข้อมูลดัชนี #############   
 
@@ -255,10 +268,14 @@ def listShareholders_Preprocess(symbol,eventdate):
   dsShares  = []
   dsPercent = []
   
+  if(len(k)==0):
+    return 0
    
   if("ไม่มีข้อมูล" in k[0].text):
     return 0
 
+ 
+
   for i in k[0].find_all("tr"):
    tr = i.find_all("td")
    name    = (tr[1].text.strip())
@@ -270,24 +287,55 @@ def listShareholders_Preprocess(symbol,eventdate):
    dsShares.append(float(Shares))
    dsPercent.append(float(Percent))
     
-  
+   
   df = pd.DataFrame({"Symbol":dsSymbol,"Name":dsName,"Share":dsShares,"Percent":dsPercent}) 
-
+   
   if(eventdate==True):
-    k = soup.find_all("tr",{"class":"topicbg"})
-    d = k[1].td.text.replace("\xa0","").split("วันที่")[1].split("ประเภท:")
-    #d[0],d[1]
-    if(len(d)==2): #กรณีไม่มีประเภทของการปิดสมุดส่งมา
-      k_date = d[0]
-      k_type = d[1]
+
+    if(len(symbol)>=3): #เช่น S-R สั้นสุดคือ ชื่อหุ้น 1 ตัวอักษร
+       twoLastSymbol = symbol[-2]+symbol[-1]
     else:
-      k_date = d[0]
-      k_type = "n/a:"+d[0]
+       twoLastSymbol = ""
+
+    if("-R" == twoLastSymbol): #กรณี ส่ง -R เข้ามา
+       
+      dttmp = soup.find_all("table",{"class":"table table-hover table-info-wrap"})   
+      dttmp = (dttmp[0].find_all("caption")[0].text)
+      k_date = dttmp.replace("\xa0","").split("วันที่")[1]
+      tmp = k_date.split("/")
+      date = str(int(tmp[2])-543)+"-"+str(tmp[1])+"-"+str(tmp[0])
+      df["Date"] = pd.to_datetime(date)
+
+      symbol_no_R = symbol[0:len(symbol)-2]
+
+      dfTypeDate = listShareholders(symbol_no_R,eventdate=True)
+      TypeDate = dfTypeDate["Type"].iloc[0]
+
+      if("n/a" in TypeDate):
+         TypeDate = "n/a"
+
+      df["Type"] = TypeDate
+
+    else:    
+      k = soup.find_all("tr",{"class":"topicbg"})
+     
+      d = k[1].td.text.replace("\xa0","").split("วันที่")[1].split("ประเภท:")
     
-    tmp = k_date.split("/")
-    date = str(int(tmp[2])-543)+"-"+str(tmp[1])+"-"+str(tmp[0])
-    df["Date"] = pd.to_datetime(date)
-    df["Type"] = k_type
+      #d[0],d[1]
+      if(len(d)==2): #กรณีไม่มีประเภทของการปิดสมุดส่งมา
+       k_date = d[0]
+       k_type = d[1]
+      else:
+       k_date = d[0]
+       k_type = "n/a:"+d[0]
+    
+      tmp = k_date.split("/")
+      date = str(int(tmp[2])-543)+"-"+str(tmp[1])+"-"+str(tmp[0])
+      df["Date"] = pd.to_datetime(date)
+
+      if("n/a" in k_type):
+         k_type = "n/a"
+      df["Type"] = k_type
 
   pd.options.display.float_format = '{:,.2f}'.format  
   return df
@@ -349,18 +397,50 @@ setattr(pd.core.frame.DataFrame, 'findPathShareholders', findPathShareholders)
 
  
 #%config InlineBackend.figure_format ='retina'
-def RelativeStrength_hist(dateStart,setUni,plotState):
- symbol = ["tdex"] 
- qs = crateQueryString(symbol)
- tdex = loaddata.get(qs,start=dateStart) #จะใช้ startDate เป็นจุดเริ่มต้น และ Dateปัจจุบันเป็น EndDate
+def RelativeStrength_hist(dateStart,dateStop,setUni,plotState,sp_data=None,numColPerRow=10,market=None):
+ '''
+ ตัวอย่าง
+ df = sx.RelativeStrength_hist(dateStart="2020-07-01",
+                              dateStop="2020-07-17",setUni=sx.indexMarket.SET100,
+                              plotState=True,sp_data=["JMART"])
+
+ dateStart : (str) วันที่เริ่มต้น
+ dateStop : (str) วันที่จบการเทรด
+ setUni : (indexMarket) เช่น set50, set100 ถ้าเป็น set อาจจะกินเวลานานมาก
+ sp_date : (list) กรณีอยากเสริมหุ้นที่อยู่นอกเหนือดัชนี      
+ plotState : (Boolean) True/False แสดงผลกราฟหรือไม่แสดง  
+ numColPerRow : (int) จำนวนแถวที่แสดงข้อมูล ถ้า plotState เป็น True                        
+ ''' 
+ #symbol = ["tdex"] 
+ #qs = crateQueryString(symbol)
+ #tdex = loaddata.get(qs,start=dateStart) #จะใช้ startDate เป็นจุดเริ่มต้น และ Dateปัจจุบันเป็น EndDate
+ #tdex.columns = ['close']
+
+ if(market==None):
+   market = setUni
+
+ symbol = "^"+market.value
+ #print(symbol)
+ tdex = loadHistData(symbol,start=dateStart,end=dateStop) 
  tdex.columns = ['close']
-    
+ 
+
  tdexbusket = pd.DataFrame({"startDate":tdex.iloc[0],"endDate":tdex.iloc[-1],"symbol":"tdex"})
  tdexbusket = tdexbusket.set_index('symbol')  
-    
+ 
  #ดึงรายชื่อหุ้นใน set50 หรือ set100  
- busketDS = getMemberOfIndex(sx.indexMarket.SET50.value)   
-    
+ #busketDS = getMemberOfIndex(indexMarket.SET50)   
+ busketDS = getMemberOfIndex(setUni)
+ #print(type(busketDS))
+
+ if(sp_data!=None):
+   key_index = np.array(range(len(busketDS),len(busketDS)+len(sp_data)))
+   new_data = pd.DataFrame({"symbol":sp_data},index=key_index)
+   busketDS  = busketDS.append(new_data)
+
+
+ #print(busketDS)
+ #return 0
  busket = pd.DataFrame()
  busketDS = np.array(busketDS)
  cnt = len(busketDS)
@@ -372,7 +452,7 @@ def RelativeStrength_hist(dateStart,setUni,plotState):
     else:
       print(symbol,end="")  
     
-    df = loaddata.get(qs,start=dateStart)
+    df = loaddata.get(qs,start=dateStart,end=dateStop)
     df.columns = [symbol]
     
     if(i==0):
@@ -390,15 +470,24 @@ def RelativeStrength_hist(dateStart,setUni,plotState):
     beginStockPrice = busket[busket.columns[j]].iloc[0]
     endStockPrice = busket[busket.columns[j]].iloc[i]
     
+    #print(tdex)
+    #return 0
+    
     beginIndex = tdex.iloc[0].values[0]
     endIndex = tdex.iloc[i].values[0]
-    #print(tdex.index[i])
+
+    #print(endStockPrice,beginStockPrice,beginIndex,endIndex)
+    #print(beginIndex,endIndex)
+    #return 0
+
     rs = ((endStockPrice/beginStockPrice) /  (endIndex/beginIndex))
     rs_ds.append(rs)
+
   if(i==1):
    tmpRS = pd.DataFrame(rs_ds).T
    tmpRS.columns = busket.columns
    tmpRS['Date'] = tdex.index[i]
+
    tmpRSX = tmpRS.set_index("Date")  
   else:
    tmpRS = pd.DataFrame(rs_ds).T
@@ -406,11 +495,14 @@ def RelativeStrength_hist(dateStart,setUni,plotState):
    tmpRS['Date'] = tdex.index[i]
    tmpRS = tmpRS.set_index("Date") 
    tmpRSX = tmpRSX.append(tmpRS) 
+
  print("..",end="")
  print(" End Process")   
 
+ 
  tmpRSX_rank = pd.DataFrame()
  for i in range(len(tmpRSX)):
+
   t = pd.DataFrame(tmpRSX.iloc[i].sort_values(ascending=False))
   t['RANK'] = list(range(len(t), 0,-1))
   t.columns = ['RS','RANK']
@@ -436,17 +528,30 @@ def RelativeStrength_hist(dateStart,setUni,plotState):
    tmpRS_rank = tmpRS_rank.set_index("Date") 
 
    tmpRSX_rank = tmpRSX_rank.append(tmpRS_rank)  
-    
+
+
+
+ from pylab import rcParams
+ rcParams['figure.figsize'] = 50, 50   
+ num_row = int(np.ceil(len(busketDS)/numColPerRow))
+ #print(num_row,numColPerRow)  
+
  if(plotState):   
-  fig, axes = plt.subplots(nrows=10, ncols=5)
+  fig, axes = plt.subplots(nrows=num_row, ncols=numColPerRow)
   k = 0
-  for i in range(10):
-   for j in range(5):  
+  for i in range(num_row):
+   for j in range(numColPerRow):  
     symbol = tmpRSX_rank.columns[k]
-    ax = tmpRSX_rank[symbol].plot(ax=axes[i,j],figsize=(20,30),title=symbol) 
-    ax.get_xaxis().set_visible(False)
-    k=k+1    
-    
+    ax = tmpRSX_rank[symbol].plot(ax=axes[i,j],figsize=(30,30),title=symbol) 
+    axes[i,j].axhline(90,color='red',ls='--')
+    axes[i,j].axhline(95,color='red',ls='--')
+    axes[i,j].set_ylim(0,100)
+    ax.get_xaxis().set_visible(False)    
+    k=k+1      
+    if(k==len(busketDS)): #ตรวจสอบความยาวของ Uni 
+       break
+
+ tmpRSX_rank.index = tmpRSX_rank.index.date #เทคนิค เอาเวลาออกจาก date คือตัด HH:MM:SS ออก   
  return tmpRSX_rank  
 
 #หาอัตราการเปรียบเปลี่ยนราคาเปรียบเทียบกับดัชนี
@@ -459,23 +564,37 @@ def RelativeStrength_hist(dateStart,setUni,plotState):
 
 
 def RelativeStrengthRank(dateStart,dateStop,setUni,dataset=0,viewlog=False):
+  '''
+  dateStart : (str) "YYYY-MM-DD"
+  dateStop : (str) "YYYY-MM-DD"
+  setUni : (str) เช่น "set","set50","set100"
+  '''
   #เตรียมส่วนของดัชนี  
   setUni = setUni.lower() #ป้องกันคนกรอกมาเป็นตัวใหญ่
   if(setUni=="set50"):
-    symbol = ["tdex"] 
+    #symbol = ["tdex"] 
+    symbol = ["^set50"] 
     setUni = indexMarket.SET50
   elif(setUni=="set100"):
-    symbol = ["bset100"]
+    #symbol = ["bset100"]
+    symbol = ["^set100"]
     setUni = indexMarket.SET100
   elif(setUni=="set"):
-    symbol = ["tdex"]
+    #symbol = ["tdex"]
+    symbol = ["^set"]
     #setUni = indexMarket.SET
   elif(setUni=="join"):
     symbol = ["tdex"]
 
   
   qs = crateQueryString(symbol)
-  ETFIndex = loaddata.get(qs,start=dateStart,end=dateStop) #จะใช้ startDate เป็นจุดเริ่มต้น และ Dateปัจจุบันเป็น EndDate
+  
+  #print(f"loadHistDataIndex({symbol},start='{dateStart}')",symbol,dateStart)
+  #return 0
+  #ETFIndex = loaddata.get(qs,start=dateStart,end=dateStop) #จะใช้ startDate เป็นจุดเริ่มต้น และ Dateปัจจุบันเป็น EndDate
+  ETFIndex = loadHistData(symbol,start=dateStart,end=dateStop)
+   
+
   ETFIndex.columns = ['close']  
   
   ETFbusket = pd.DataFrame({"startDate":ETFIndex.iloc[0],"endDate":ETFIndex.iloc[-1],"symbol":symbol})
@@ -499,6 +618,7 @@ def RelativeStrengthRank(dateStart,dateStop,setUni,dataset=0,viewlog=False):
     print("Process Data...",end="")
   for i in range(cnt):
     symbol = busketDS[i][0]
+    symbol = symbol.replace("&","%26") 
     qs = crateQueryString([symbol])
     if(i<cnt-1):
       if(viewlog==True):
@@ -510,7 +630,8 @@ def RelativeStrengthRank(dateStart,dateStop,setUni,dataset=0,viewlog=False):
     try:
      df = loaddata.get(qs,start=dateStart,end=dateStop)
      df.columns = ['close']
-    
+     print(symbol,end=" ")
+
      k1 = pd.DataFrame({"startDate":df.iloc[0],"endDate":df.iloc[-1],"symbol":symbol}) 
      k1 = k1.set_index('symbol')
      busket = busket.append(k1) 
@@ -588,7 +709,11 @@ def MonthlyReturn(symbol,startDate,endDate="",plot=True,OutPutfilename=""):
      now = datetime.datetime.now()
      endDate = (str(now.year).strip()+"-"+str(now.month).strip()+"-"+str(now.day).strip())   
   
-  df = loaddata.get(qs, start=startDate,end=endDate)
+  if("^" in qs):
+    df = loadHistData(symbol,start=startDate,end=endDate) 
+  else:
+    df = loaddata.get(qs, start=startDate,end=endDate)
+  
   df.columns = symbol
   df = df.rebase()
     
@@ -1021,12 +1146,15 @@ def trimstring2float(s):
     s = s.replace(",","")
     return s
 
-def setTHforPlot():
+def setTHforPlot(fontsize=20.0):
    #set title ภาษาไทย
    #https://stackoverflow.com/questions/16574898/how-to-load-ttf-file-in-matplotlib-using-mpl-rcparams
+   mpl.rcParams['font.size'] = fontsize
    path = 'DB_Helvethaica_X.ttf'
    prop = font_manager.FontProperties(fname=path)
    mpl.rcParams['font.family'] = prop.get_name()
+    
+ 
 
 ############## spiderCompare #######################################
 def spiderCompare(liststock,plot=True):
